@@ -65,9 +65,6 @@
 
 #include "gstfaceblur.h"
 #include <opencv2/imgproc.hpp>
-#if (CV_MAJOR_VERSION >= 4)
-#include <opencv2/imgproc/imgproc_c.h>
-#endif
 
 GST_DEBUG_CATEGORY_STATIC (gst_face_blur_debug);
 #define GST_CAT_DEFAULT gst_face_blur_debug
@@ -156,10 +153,10 @@ static void gst_face_blur_get_property (GObject * object, guint prop_id,
 
 
 static gboolean gst_face_blur_set_caps (GstOpencvVideoFilter * transform,
-    gint in_width, gint in_height, gint in_depth, gint in_channels,
-    gint out_width, gint out_height, gint out_depth, gint out_channels);
+    gint in_width, gint in_height, int in_cv_type,
+    gint out_width, gint out_height, int out_cv_type);
 static GstFlowReturn gst_face_blur_transform_ip (GstOpencvVideoFilter *
-    transform, GstBuffer * buffer, IplImage * img);
+    transform, GstBuffer * buffer, Mat img);
 
 static CascadeClassifier *gst_face_blur_load_profile (GstFaceBlur *
     filter, gchar * profile);
@@ -170,8 +167,7 @@ gst_face_blur_finalize (GObject * obj)
 {
   GstFaceBlur *filter = GST_FACE_BLUR (obj);
 
-  if (filter->cvGray)
-    cvReleaseImage (&filter->cvGray);
+  filter->cvGray.release ();
 
   if (filter->cvCascade)
     delete filter->cvCascade;
@@ -328,22 +324,19 @@ gst_face_blur_get_property (GObject * object, guint prop_id,
 
 static gboolean
 gst_face_blur_set_caps (GstOpencvVideoFilter * transform,
-    gint in_width, gint in_height, gint in_depth, gint in_channels,
-    gint out_width, gint out_height, gint out_depth, gint out_channels)
+    gint in_width, gint in_height, int in_cv_type,
+    gint out_width, gint out_height, int out_cv_type)
 {
   GstFaceBlur *filter = GST_FACE_BLUR (transform);
 
-  if (filter->cvGray)
-    cvReleaseImage (&filter->cvGray);
-  filter->cvGray =
-      cvCreateImage (cvSize (in_width, in_height), IPL_DEPTH_8U, 1);
+  filter->cvGray.create (Size (in_width, in_height), CV_8UC1);
 
   return TRUE;
 }
 
 static GstFlowReturn
 gst_face_blur_transform_ip (GstOpencvVideoFilter * transform,
-    GstBuffer * buffer, IplImage * img)
+    GstBuffer * buffer, Mat img)
 {
   GstFaceBlur *filter = GST_FACE_BLUR (transform);
   vector < Rect > faces;
@@ -360,19 +353,17 @@ gst_face_blur_transform_ip (GstOpencvVideoFilter * transform,
     return GST_FLOW_OK;
   }
 
-  cvCvtColor (img, filter->cvGray, CV_RGB2GRAY);
+  cvtColor (img, filter->cvGray, COLOR_RGB2GRAY);
 
-  Mat image = cvarrToMat (filter->cvGray);
-  filter->cvCascade->detectMultiScale (image, faces, filter->scale_factor,
-      filter->min_neighbors, filter->flags,
-      cvSize (filter->min_size_width, filter->min_size_height), cvSize (0, 0));
+  filter->cvCascade->detectMultiScale (filter->cvGray, faces,
+      filter->scale_factor, filter->min_neighbors, filter->flags,
+      Size (filter->min_size_width, filter->min_size_height), Size (0, 0));
 
   if (!faces.empty ()) {
 
     for (i = 0; i < faces.size (); ++i) {
       Rect *r = &faces[i];
-      Mat imag = cvarrToMat (img);
-      Mat roi (imag, Rect (r->x, r->y, r->width, r->height));
+      Mat roi (img, Rect (r->x, r->y, r->width, r->height));
       blur (roi, roi, Size (11, 11));
       GaussianBlur (roi, roi, Size (11, 11), 0, 0);
     }
