@@ -63,9 +63,6 @@
 /* element header */
 #include "gsthanddetect.h"
 #include <opencv2/imgproc.hpp>
-#if (CV_MAJOR_VERSION >= 4)
-#include <opencv2/imgproc/imgproc_c.h>
-#endif
 
 GST_DEBUG_CATEGORY_STATIC (gst_handdetect_debug);
 #define GST_CAT_DEFAULT gst_handdetect_debug
@@ -115,10 +112,10 @@ static void gst_handdetect_set_property (GObject * object, guint prop_id,
 static void gst_handdetect_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static gboolean gst_handdetect_set_caps (GstOpencvVideoFilter * transform,
-    gint in_width, gint in_height, gint in_depth, gint in_channels,
-    gint out_width, gint out_height, gint out_depth, gint out_channels);
+    gint in_width, gint in_height, int in_cv_type,
+    gint out_width, gint out_height, int out_cv_type);
 static GstFlowReturn gst_handdetect_transform_ip (GstOpencvVideoFilter *
-    transform, GstBuffer * buffer, IplImage * img);
+    transform, GstBuffer * buffer, Mat img);
 
 static CascadeClassifier *gst_handdetect_load_profile (GstHanddetect * filter,
     gchar * profile);
@@ -165,11 +162,14 @@ gst_handdetect_finalize (GObject * obj)
 {
   GstHanddetect *filter = GST_HANDDETECT (obj);
 
-  if (filter->cvGray)
-    cvReleaseImage (&filter->cvGray);
+  filter->cvGray.release ();
   g_free (filter->profile_fist);
   g_free (filter->profile_palm);
   delete (filter->best_r);
+  if (filter->cvCascade_fist)
+    delete filter->cvCascade_fist;
+  if (filter->cvCascade_palm)
+    delete filter->cvCascade_palm;
 
   G_OBJECT_CLASS (gst_handdetect_parent_class)->finalize (obj);
 }
@@ -373,8 +373,8 @@ gst_handdetect_get_property (GObject * object, guint prop_id, GValue * value,
 /* this function handles the link with other elements */
 static gboolean
 gst_handdetect_set_caps (GstOpencvVideoFilter * transform,
-    gint in_width, gint in_height, gint in_depth, gint in_channels,
-    gint out_width, gint out_height, gint out_depth, gint out_channels)
+    gint in_width, gint in_height, int in_cv_type,
+    gint out_width, gint out_height, int out_cv_type)
 {
   GstHanddetect *filter;
   filter = GST_HANDDETECT (transform);
@@ -384,10 +384,7 @@ gst_handdetect_set_caps (GstOpencvVideoFilter * transform,
     GST_WARNING_OBJECT (filter,
         "resize to 320 x 240 to have best detect accuracy.\n");
 
-  if (filter->cvGray)
-    cvReleaseImage (&filter->cvGray);
-  filter->cvGray =
-      cvCreateImage (cvSize (in_width, in_height), IPL_DEPTH_8U, 1);
+  filter->cvGray.create (Size (in_width, in_height), CV_8UC1);
 
   return TRUE;
 }
@@ -397,7 +394,7 @@ gst_handdetect_set_caps (GstOpencvVideoFilter * transform,
  */
 static GstFlowReturn
 gst_handdetect_transform_ip (GstOpencvVideoFilter * transform,
-    GstBuffer * buffer, IplImage * img)
+    GstBuffer * buffer, Mat img)
 {
   GstHanddetect *filter = GST_HANDDETECT (transform);
   Rect *r;
@@ -409,29 +406,27 @@ gst_handdetect_transform_ip (GstOpencvVideoFilter * transform,
   /* check detection cascades */
   if (filter->cvCascade_fist && filter->cvCascade_palm) {
     /* cvt to gray colour space for hand detect */
-    cvCvtColor (img, filter->cvGray, CV_RGB2GRAY);
+    cvtColor (img, filter->cvGray, COLOR_RGB2GRAY);
 
     /* detect FIST gesture fist */
-    Mat image = cvarrToMat (filter->cvGray);
-    Mat roi (image, Rect (filter->cvGray->origin,
-            filter->cvGray->origin, filter->cvGray->width,
-            filter->cvGray->height));
+    Mat roi (filter->cvGray, Rect (0,
+            0, filter->cvGray.size ().width, filter->cvGray.size ().height));
     filter->cvCascade_fist->detectMultiScale (roi, hands, 1.1, 2,
-        CASCADE_DO_CANNY_PRUNING, cvSize (24, 24), cvSize (0, 0));
+        CASCADE_DO_CANNY_PRUNING, Size (24, 24), Size (0, 0));
 
     /* if FIST gesture detected */
     if (!hands.empty ()) {
 
       int min_distance, distance;
       Rect temp_r;
-      CvPoint c;
+      Point c;
 
       /* Go through all detected FIST gestures to get the best one
        * prev_r => previous hand
        * best_r => best hand in this frame
        */
       /* set min_distance for init comparison */
-      min_distance = img->width + img->height;
+      min_distance = img.size ().width + img.size ().height;
       /* Init filter->prev_r */
       temp_r = Rect (0, 0, 0, 0);
       if (filter->prev_r == NULL)
@@ -452,7 +447,7 @@ gst_handdetect_transform_ip (GstOpencvVideoFilter * transform,
 
       /* send msg to app/bus if the detected gesture falls in the region of interest */
       /* get center point of gesture */
-      c = cvPoint (filter->best_r->x + filter->best_r->width / 2,
+      c = Point (filter->best_r->x + filter->best_r->width / 2,
           filter->best_r->y + filter->best_r->height / 2);
       /* send message:
        * if the center point is in the region of interest, OR,
@@ -494,28 +489,23 @@ gst_handdetect_transform_ip (GstOpencvVideoFilter * transform,
       /* Check filter->display,
        * If TRUE, displaying red circle marker in the out frame */
       if (filter->display) {
-        CvPoint center;
+        Point center;
         int radius;
         center.x = cvRound ((filter->best_r->x + filter->best_r->width * 0.5));
         center.y = cvRound ((filter->best_r->y + filter->best_r->height * 0.5));
         radius =
             cvRound ((filter->best_r->width + filter->best_r->height) * 0.25);
-        cvCircle (img, center, radius, CV_RGB (0, 0, 200), 1, 8, 0);
+        circle (img, center, radius, CV_RGB (0, 0, 200), 1, 8, 0);
       }
     } else {
       /* if NO FIST gesture, detecting PALM gesture */
-#if (CV_MAJOR_VERSION >= 4)
       filter->cvCascade_palm->detectMultiScale (roi, hands, 1.1, 2,
-          CASCADE_DO_CANNY_PRUNING, cvSize (24, 24), cvSize (0, 0));
-#else
-      filter->cvCascade_palm->detectMultiScale (roi, hands, 1.1, 2,
-          CV_HAAR_DO_CANNY_PRUNING, cvSize (24, 24), cvSize (0, 0));
-#endif
+          CASCADE_DO_CANNY_PRUNING, Size (24, 24), Size (0, 0));
       /* if PALM detected */
       if (!hands.empty ()) {
         int min_distance, distance;
         Rect temp_r;
-        CvPoint c;
+        Point c;
 
         if (filter->display) {
           GST_DEBUG_OBJECT (filter, "%d PALM gestures detected\n",
@@ -526,7 +516,7 @@ gst_handdetect_transform_ip (GstOpencvVideoFilter * transform,
          * best_r => best hand in this frame
          */
         /* suppose a min_distance for init comparison */
-        min_distance = img->width + img->height;
+        min_distance = img.size ().width + img.size ().height;
         /* Init filter->prev_r */
         temp_r = Rect (0, 0, 0, 0);
         if (filter->prev_r == NULL)
@@ -547,7 +537,7 @@ gst_handdetect_transform_ip (GstOpencvVideoFilter * transform,
 
         /* send msg to app/bus if the detected gesture falls in the region of interest */
         /* get center point of gesture */
-        c = cvPoint (filter->best_r->x + filter->best_r->width / 2,
+        c = Point (filter->best_r->x + filter->best_r->width / 2,
             filter->best_r->y + filter->best_r->height / 2);
         /* send message:
          * if the center point is in the region of interest, OR,
@@ -603,7 +593,7 @@ gst_handdetect_transform_ip (GstOpencvVideoFilter * transform,
         /* Check filter->display,
          * If TRUE, displaying red circle marker in the out frame */
         if (filter->display) {
-          CvPoint center;
+          Point center;
           int radius;
           center.x =
               cvRound ((filter->best_r->x + filter->best_r->width * 0.5));
@@ -611,7 +601,7 @@ gst_handdetect_transform_ip (GstOpencvVideoFilter * transform,
               cvRound ((filter->best_r->y + filter->best_r->height * 0.5));
           radius =
               cvRound ((filter->best_r->width + filter->best_r->height) * 0.25);
-          cvCircle (img, center, radius, CV_RGB (0, 0, 200), 1, 8, 0);
+          circle (img, center, radius, CV_RGB (0, 0, 200), 1, 8, 0);
         }
       }
     }
