@@ -1,7 +1,7 @@
 /*
- * gstmpegtsdescriptor.h - 
+ * gstmpegtsdescriptor.h -
  * Copyright (C) 2013 Edward Hervey
- * 
+ *
  * Authors:
  *   Edward Hervey <edward@collabora.com>
  *
@@ -243,7 +243,7 @@ struct _GstMpegtsSatelliteDeliverySystemDescriptor
 {
   guint32                            frequency;
   gfloat                             orbital_position;
-  gboolean                           west_east; 
+  gboolean                           west_east;
   GstMpegtsSatellitePolarizationType polarization;
 
   GstMpegtsSatelliteRolloff          roll_off;
@@ -643,6 +643,11 @@ struct _GstMpegtsContent
   guint8 user_byte;
 };
 
+#define GST_TYPE_MPEGTS_CONTENT (gst_mpegts_content_get_type ())
+
+GST_MPEGTS_API
+GType gst_mpegts_content_get_type (void);
+
 GST_MPEGTS_API
 gboolean gst_mpegts_descriptor_parse_dvb_content (const GstMpegtsDescriptor *
 	descriptor, GPtrArray ** content);
