@@ -20,6 +20,7 @@
 
 /**
  * SECTION:gstplayer-mediainfo
+ * @title: GstPlayerMediaInfo
  * @short_description: Player Media Information
  *
  */
@@ -47,6 +48,7 @@ gst_player_stream_info_finalize (GObject * object)
   GstPlayerStreamInfo *sinfo = GST_PLAYER_STREAM_INFO (object);
 
   g_free (sinfo->codec);
+  g_free (sinfo->stream_id);
 
   if (sinfo->caps)
     gst_caps_unref (sinfo->caps);
@@ -543,6 +545,8 @@ gst_player_stream_info_copy (GstPlayerStreamInfo * ref)
     info->caps = gst_caps_copy (ref->caps);
   if (ref->codec)
     info->codec = g_strdup (ref->codec);
+  if (ref->stream_id)
+    info->stream_id = g_strdup (ref->stream_id);
 
   return info;
 }
@@ -675,14 +679,14 @@ gst_player_media_info_get_stream_list (const GstPlayerMediaInfo * info)
 }
 
 /**
- * gst_player_get_video_streams:
+ * gst_player_media_info_get_video_streams:
  * @info: a #GstPlayerMediaInfo
  *
  * Returns: (transfer none) (element-type GstPlayerVideoInfo): A #GList of
  * matching #GstPlayerVideoInfo.
  */
 GList *
-gst_player_get_video_streams (const GstPlayerMediaInfo * info)
+gst_player_media_info_get_video_streams (const GstPlayerMediaInfo * info)
 {
   g_return_val_if_fail (GST_IS_PLAYER_MEDIA_INFO (info), NULL);
 
@@ -690,14 +694,14 @@ gst_player_get_video_streams (const GstPlayerMediaInfo * info)
 }
 
 /**
- * gst_player_get_subtitle_streams:
+ * gst_player_media_info_get_subtitle_streams:
  * @info: a #GstPlayerMediaInfo
  *
  * Returns: (transfer none) (element-type GstPlayerSubtitleInfo): A #GList of
  * matching #GstPlayerSubtitleInfo.
  */
 GList *
-gst_player_get_subtitle_streams (const GstPlayerMediaInfo * info)
+gst_player_media_info_get_subtitle_streams (const GstPlayerMediaInfo * info)
 {
   g_return_val_if_fail (GST_IS_PLAYER_MEDIA_INFO (info), NULL);
 
@@ -705,14 +709,14 @@ gst_player_get_subtitle_streams (const GstPlayerMediaInfo * info)
 }
 
 /**
- * gst_player_get_audio_streams:
+ * gst_player_media_info_get_audio_streams:
  * @info: a #GstPlayerMediaInfo
  *
  * Returns: (transfer none) (element-type GstPlayerAudioInfo): A #GList of
  * matching #GstPlayerAudioInfo.
  */
 GList *
-gst_player_get_audio_streams (const GstPlayerMediaInfo * info)
+gst_player_media_info_get_audio_streams (const GstPlayerMediaInfo * info)
 {
   g_return_val_if_fail (GST_IS_PLAYER_MEDIA_INFO (info), NULL);
 
@@ -791,3 +795,110 @@ gst_player_media_info_get_image_sample (const GstPlayerMediaInfo * info)
 
   return info->image_sample;
 }
+
+/**
+ * gst_player_media_info_get_number_of_streams:
+ * @info: a #GstPlayerMediaInfo
+ *
+ * Returns: number of total streams.
+ * Since: 1.12
+ */
+guint
+gst_player_media_info_get_number_of_streams (const GstPlayerMediaInfo * info)
+{
+  g_return_val_if_fail (GST_IS_PLAYER_MEDIA_INFO (info), 0);
+
+  return g_list_length (info->stream_list);
+}
+
+/**
+ * gst_player_media_info_get_number_of_video_streams:
+ * @info: a #GstPlayerMediaInfo
+ *
+ * Returns: number of video streams.
+ * Since: 1.12
+ */
+guint
+gst_player_media_info_get_number_of_video_streams (const GstPlayerMediaInfo *
+    info)
+{
+  g_return_val_if_fail (GST_IS_PLAYER_MEDIA_INFO (info), 0);
+
+  return g_list_length (info->video_stream_list);
+}
+
+/**
+ * gst_player_media_info_get_number_of_audio_streams:
+ * @info: a #GstPlayerMediaInfo
+ *
+ * Returns: number of audio streams.
+ * Since: 1.12
+ */
+guint
+gst_player_media_info_get_number_of_audio_streams (const GstPlayerMediaInfo *
+    info)
+{
+  g_return_val_if_fail (GST_IS_PLAYER_MEDIA_INFO (info), 0);
+
+  return g_list_length (info->audio_stream_list);
+}
+
+/**
+ * gst_player_media_info_get_number_of_subtitle_streams:
+ * @info: a #GstPlayerMediaInfo
+ *
+ * Returns: number of subtitle streams.
+ * Since: 1.12
+ */
+guint gst_player_media_info_get_number_of_subtitle_streams
+    (const GstPlayerMediaInfo * info)
+{
+  g_return_val_if_fail (GST_IS_PLAYER_MEDIA_INFO (info), 0);
+
+  return g_list_length (info->subtitle_stream_list);
+}
+
+/**
+ * gst_player_get_video_streams:
+ * @info: a #GstPlayerMediaInfo
+ *
+ * Returns: (transfer none) (element-type GstPlayerVideoInfo): A #GList of
+ * matching #GstPlayerVideoInfo.
+ */
+#ifndef GST_REMOVE_DEPRECATED
+GList *
+gst_player_get_video_streams (const GstPlayerMediaInfo * info)
+{
+  return gst_player_media_info_get_video_streams (info);
+}
+#endif
+
+/**
+ * gst_player_get_audio_streams:
+ * @info: a #GstPlayerMediaInfo
+ *
+ * Returns: (transfer none) (element-type GstPlayerAudioInfo): A #GList of
+ * matching #GstPlayerAudioInfo.
+ */
+#ifndef GST_REMOVE_DEPRECATED
+GList *
+gst_player_get_audio_streams (const GstPlayerMediaInfo * info)
+{
+  return gst_player_media_info_get_audio_streams (info);
+}
+#endif
+
+/**
+ * gst_player_get_subtitle_streams:
+ * @info: a #GstPlayerMediaInfo
+ *
+ * Returns: (transfer none) (element-type GstPlayerSubtitleInfo): A #GList of
+ * matching #GstPlayerSubtitleInfo.
+ */
+#ifndef GST_REMOVE_DEPRECATED
+GList *
+gst_player_get_subtitle_streams (const GstPlayerMediaInfo * info)
+{
+  return gst_player_media_info_get_subtitle_streams (info);
+}
+#endif
