@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -137,7 +137,7 @@ gst_frei0r_klass_install_properties (GObjectClass * gobject_class,
         if (!(def <= 1.0 && def >= 0.0))
           def = 0.0;
         prop_name_full = g_strconcat (prop_name, "-r", NULL);
-        prop_nick_full = g_strconcat (param_info->name, "-R", NULL);
+        prop_nick_full = g_strconcat (param_info->name, " (R)", NULL);
         g_object_class_install_property (gobject_class, count++,
             g_param_spec_float (prop_name_full, prop_nick_full,
                 param_info->explanation, 0.0, 1.0, def,
@@ -150,9 +150,9 @@ gst_frei0r_klass_install_properties (GObjectClass * gobject_class,
         if (!(def <= 1.0 && def >= 0.0))
           def = 0.0;
         prop_name_full = g_strconcat (prop_name, "-g", NULL);
-        prop_nick_full = g_strconcat (param_info->name, "-G", NULL);
+        prop_nick_full = g_strconcat (param_info->name, " (G)", NULL);
         g_object_class_install_property (gobject_class, count++,
-            g_param_spec_float (prop_name_full, param_info->name,
+            g_param_spec_float (prop_name_full, prop_nick_full,
                 param_info->explanation, 0.0, 1.0, def,
                 G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
         g_free (prop_name_full);
@@ -163,9 +163,9 @@ gst_frei0r_klass_install_properties (GObjectClass * gobject_class,
         if (!(def <= 1.0 && def >= 0.0))
           def = 0.0;
         prop_name_full = g_strconcat (prop_name, "-b", NULL);
-        prop_nick_full = g_strconcat (param_info->name, "-B", NULL);
+        prop_nick_full = g_strconcat (param_info->name, " (B)", NULL);
         g_object_class_install_property (gobject_class, count++,
-            g_param_spec_float (prop_name_full, param_info->name,
+            g_param_spec_float (prop_name_full, prop_nick_full,
                 param_info->explanation, 0.0, 1.0, def,
                 G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
         g_free (prop_name_full);
@@ -184,9 +184,9 @@ gst_frei0r_klass_install_properties (GObjectClass * gobject_class,
         if (!(def <= 1.0 && def >= 0.0))
           def = 0.0;
         prop_name_full = g_strconcat (prop_name, "-x", NULL);
-        prop_nick_full = g_strconcat (param_info->name, "-X", NULL);
+        prop_nick_full = g_strconcat (param_info->name, " (X)", NULL);
         g_object_class_install_property (gobject_class, count++,
-            g_param_spec_double (prop_name_full, param_info->name,
+            g_param_spec_double (prop_name_full, prop_nick_full,
                 param_info->explanation, 0.0, 1.0, def,
                 G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
         g_free (prop_name_full);
@@ -197,9 +197,9 @@ gst_frei0r_klass_install_properties (GObjectClass * gobject_class,
         if (!(def <= 1.0 && def >= 0.0))
           def = 0.0;
         prop_name_full = g_strconcat (prop_name, "-Y", NULL);
-        prop_nick_full = g_strconcat (param_info->name, "-X", NULL);
+        prop_nick_full = g_strconcat (param_info->name, " (Y)", NULL);
         g_object_class_install_property (gobject_class, count++,
-            g_param_spec_double (prop_name_full, param_info->name,
+            g_param_spec_double (prop_name_full, prop_nick_full,
                 param_info->explanation, 0.0, 1.0, def,
                 G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
         g_free (prop_name_full);
@@ -258,12 +258,8 @@ gst_frei0r_instance_construct (GstFrei0rFuncTable * ftable,
   f0r_instance_t *instance = ftable->construct (width, height);
   gint i;
 
-  for (i = 0; i < n_properties; i++) {
-    if (properties[i].info.type == F0R_PARAM_STRING)
-      ftable->set_param_value (instance, property_cache[i].data.s, i);
-    else
-      ftable->set_param_value (instance, &property_cache[i].data, i);
-  }
+  for (i = 0; i < n_properties; i++)
+    ftable->set_param_value (instance, &property_cache[i].data, i);
 
   return instance;
 }
@@ -311,7 +307,7 @@ gst_frei0r_get_property (f0r_instance_t * instance, GstFrei0rFuncTable * ftable,
       break;
     }
     case F0R_PARAM_STRING:{
-      const gchar *s;
+      gchar *s;
 
       if (instance)
         ftable->get_param_value (instance, &s, prop->prop_idx);
