@@ -67,7 +67,8 @@ static GstStaticPadTemplate mpegpsmux_sink_factory =
         "mpegversion = (int) { 1, 2, 4 }, "
         "systemstream = (boolean) false; "
         "video/x-dirac;"
-        "video/x-h264;"
+        "video/x-h264,stream-format=(string)byte-stream,"
+        "alignment=(string){au, nal}; "
         "audio/mpeg, "
         "mpegversion = (int) { 1, 2 };"
         "audio/mpeg, "
