@@ -26,6 +26,10 @@
  * #GstPcapParse:src-port and #GstPcapParse:dst-port to restrict which packets
  * should be included.
  *
+ * The supported data format is the classical <ulink
+ * url="https://wiki.wireshark.org/Development/LibpcapFileFormat">libpcap file
+ * format</ulink>.
+ *
  * ## Example pipelines
  * |[
  * gst-launch-1.0 filesrc location=h264crasher.pcap ! pcapparse ! rtph264depay
@@ -55,6 +59,13 @@
 #include <winsock2.h>
 #endif
 
+
+const guint GST_PCAPPARSE_MAGIC_MILLISECOND_NO_SWAP_ENDIAN = 0xa1b2c3d4;
+const guint GST_PCAPPARSE_MAGIC_NANOSECOND_NO_SWAP_ENDIAN = 0xa1b23c4d;
+const guint GST_PCAPPARSE_MAGIC_MILLISECOND_SWAP_ENDIAN = 0xd4c3b2a1;
+const guint GST_PCAPPARSE_MAGIC_NANOSECOND_SWAP_ENDIAN = 0x4d3cb2a1;
+
+
 enum
 {
   PROP_0,
@@ -309,6 +320,7 @@ gst_pcap_parse_reset (GstPcapParse * self)
 {
   self->initialized = FALSE;
   self->swap_endian = FALSE;
+  self->nanosecond_timestamp = FALSE;
   self->cur_packet_size = -1;
   self->cur_ts = GST_CLOCK_TIME_NONE;
   self->base_ts = GST_CLOCK_TIME_NONE;
@@ -333,7 +345,9 @@ gst_pcap_parse_read_uint32 (GstPcapParse * self, const guint8 * p)
   }
 }
 
+#define ETH_MAC_ADDRESSES_LEN    12
 #define ETH_HEADER_LEN    14
+#define ETH_VLAN_HEADER_LEN    4
 #define SLL_HEADER_LEN    16
 #define IP_HEADER_MIN_LEN 20
 #define UDP_HEADER_LEN     8
@@ -352,6 +366,8 @@ gst_pcap_parse_scan_frame (GstPcapParse * self,
   guint16 eth_type;
   guint8 b;
   guint8 ip_header_size;
+  guint8 flags;
+  guint16 fragment_offset;
   guint8 ip_protocol;
   guint32 ip_src_addr;
   guint32 ip_dst_addr;
@@ -363,9 +379,20 @@ gst_pcap_parse_scan_frame (GstPcapParse * self,
     case LINKTYPE_ETHER:
       if (buf_size < ETH_HEADER_LEN + IP_HEADER_MIN_LEN + UDP_HEADER_LEN)
         return FALSE;
-
-      eth_type = GUINT16_FROM_BE (*((guint16 *) (buf + 12)));
-      buf_ip = buf + ETH_HEADER_LEN;
+      eth_type = GUINT16_FROM_BE (*((guint16 *) (buf + ETH_MAC_ADDRESSES_LEN)));
+      /* check for vlan 802.1q header (4 bytes, with first two bytes equal to 0x8100)  */
+      if (eth_type == 0x8100) {
+        if (buf_size <
+            ETH_HEADER_LEN + ETH_VLAN_HEADER_LEN + IP_HEADER_MIN_LEN +
+            UDP_HEADER_LEN)
+          return FALSE;
+        eth_type =
+            GUINT16_FROM_BE (*((guint16 *) (buf + ETH_MAC_ADDRESSES_LEN +
+                    ETH_VLAN_HEADER_LEN)));
+        buf_ip = buf + ETH_HEADER_LEN + ETH_VLAN_HEADER_LEN;
+      } else {
+        buf_ip = buf + ETH_HEADER_LEN;
+      }
       break;
     case LINKTYPE_SLL:
       if (buf_size < SLL_HEADER_LEN + IP_HEADER_MIN_LEN + UDP_HEADER_LEN)
@@ -386,10 +413,16 @@ gst_pcap_parse_scan_frame (GstPcapParse * self,
       return FALSE;
   }
 
-  if (eth_type != 0x800)
+  if (eth_type != 0x800) {
+    GST_ERROR_OBJECT (self,
+        "Link type %d: Ethernet type %d is not supported; only type 0x800",
+        (gint) self->linktype, (gint) eth_type);
     return FALSE;
+  }
 
   b = *buf_ip;
+
+  /* Check that the packet is IPv4 */
   if (((b >> 4) & 0x0f) != 4)
     return FALSE;
 
@@ -397,6 +430,14 @@ gst_pcap_parse_scan_frame (GstPcapParse * self,
   if (buf_ip + ip_header_size > buf + buf_size)
     return FALSE;
 
+  flags = buf_ip[6] >> 5;
+  fragment_offset =
+      (GUINT16_FROM_BE (*((guint16 *) (buf_ip + 6))) & 0x1fff) * 8;
+  if (flags & 0x1 || fragment_offset > 0) {
+    GST_ERROR_OBJECT (self, "Fragmented packets are not supported");
+    return FALSE;
+  }
+
   ip_protocol = *(buf_ip + 9);
   GST_LOG_OBJECT (self, "ip proto %d", (gint) ip_protocol);
 
@@ -465,6 +506,7 @@ gst_pcap_parse_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 
     if (self->initialized) {
       if (self->cur_packet_size >= 0) {
+        /* Parse the Packet Data */
         if (avail < self->cur_packet_size)
           break;
 
@@ -518,10 +560,12 @@ gst_pcap_parse_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 
         self->cur_packet_size = -1;
       } else {
+        /* Parse the Record (Packet) Header */
         guint32 ts_sec;
         guint32 ts_usec;
         guint32 incl_len;
 
+        /* sizeof(pcaprec_hdr_t) == 16 */
         if (avail < 16)
           break;
 
@@ -535,14 +579,18 @@ gst_pcap_parse_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
         gst_adapter_unmap (self->adapter);
         gst_adapter_flush (self->adapter, 16);
 
-        self->cur_ts = ts_sec * GST_SECOND + ts_usec * GST_USECOND;
+        self->cur_ts =
+            ts_sec * GST_SECOND +
+            ts_usec * (self->nanosecond_timestamp ? 1 : GST_USECOND);
         self->cur_packet_size = incl_len;
       }
     } else {
+      /* Parse the Global Header */
       guint32 magic;
       guint32 linktype;
       guint16 major_version;
 
+      /* sizeof(pcap_hdr_t) == 24 */
       if (avail < 24)
         break;
 
@@ -550,14 +598,21 @@ gst_pcap_parse_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 
       magic = *((guint32 *) data);
       major_version = *((guint16 *) (data + 4));
-      linktype = gst_pcap_parse_read_uint32 (self, data + 20);
+      linktype = *((guint32 *) (data + 20));
       gst_adapter_unmap (self->adapter);
 
-      if (magic == 0xa1b2c3d4) {
+      if (magic == GST_PCAPPARSE_MAGIC_MILLISECOND_NO_SWAP_ENDIAN ||
+          magic == GST_PCAPPARSE_MAGIC_NANOSECOND_NO_SWAP_ENDIAN) {
         self->swap_endian = FALSE;
-      } else if (magic == 0xd4c3b2a1) {
+        if (magic == GST_PCAPPARSE_MAGIC_NANOSECOND_NO_SWAP_ENDIAN)
+          self->nanosecond_timestamp = TRUE;
+      } else if (magic == GST_PCAPPARSE_MAGIC_MILLISECOND_SWAP_ENDIAN ||
+          magic == GST_PCAPPARSE_MAGIC_NANOSECOND_SWAP_ENDIAN) {
         self->swap_endian = TRUE;
-        major_version = major_version << 8 | major_version >> 8;
+        if (magic == GST_PCAPPARSE_MAGIC_NANOSECOND_SWAP_ENDIAN)
+          self->nanosecond_timestamp = TRUE;
+        major_version = GUINT16_SWAP_LE_BE (major_version);
+        linktype = GUINT32_SWAP_LE_BE (linktype);
       } else {
         GST_ELEMENT_ERROR (self, STREAM, WRONG_TYPE, (NULL),
             ("File is not a libpcap file, magic is %X", magic));
