@@ -30,6 +30,7 @@
 #ifdef HAVE_AVFOUNDATION
 #include "avfvideosrc.h"
 #include "avfassetsrc.h"
+#include "avfdeviceprovider.h"
 #include "avsamplevideosink.h"
 #endif
 #ifdef HAVE_VIDEOTOOLBOX
@@ -80,6 +81,8 @@ plugin_init (GstPlugin * plugin)
       GST_TYPE_AVF_ASSET_SRC);
   res &= gst_element_register (plugin, "avsamplebufferlayersink",
       GST_RANK_NONE, GST_TYPE_AV_SAMPLE_VIDEO_SINK);
+  res &= gst_device_provider_register (plugin, "avfdeviceprovider",
+    GST_RANK_PRIMARY, GST_TYPE_AVF_DEVICE_PROVIDER);
 #endif
 
   res &= gst_element_register (plugin, "atdec", GST_RANK_MARGINAL, GST_TYPE_ATDEC);
