@@ -54,7 +54,7 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
 GST_DEBUG_CATEGORY (gst_d3dvideosink_debug);
 #define GST_CAT_DEFAULT gst_d3dvideosink_debug
 
-/** FWD DECLS **/
+/* FWD DECLS */
 /* GstXOverlay Interface */
 static void
 gst_d3dvideosink_video_overlay_interface_init (GstVideoOverlayInterface *
@@ -176,7 +176,7 @@ gst_d3dvideosink_init (GstD3DVideoSink * sink)
   g_rec_mutex_init (&sink->lock);
 }
 
-/** GObject Functions **/
+/* GObject Functions */
 
 static void
 gst_d3dvideosink_finalize (GObject * gobject)
@@ -245,7 +245,7 @@ gst_d3dvideosink_get_property (GObject * object, guint prop_id, GValue * value,
   }
 }
 
-/** GstBaseSinkClass Functions **/
+/* GstBaseSinkClass Functions */
 
 static GstCaps *
 gst_d3dvideosink_get_caps (GstBaseSink * basesink, GstCaps * filter)
@@ -470,6 +470,7 @@ gst_d3dvideosink_propose_allocation (GstBaseSink * bsink, GstQuery * query)
   return TRUE;
 #endif
 
+  /* FIXME re-using buffer pool breaks renegotiation */
   GST_OBJECT_LOCK (sink);
   pool = sink->pool ? gst_object_ref (sink->pool) : NULL;
   GST_OBJECT_UNLOCK (sink);
@@ -489,9 +490,7 @@ gst_d3dvideosink_propose_allocation (GstBaseSink * bsink, GstQuery * query)
       pool = NULL;
     }
     gst_structure_free (config);
-  }
-
-  if (pool == NULL && need_pool) {
+  } else {
     GstVideoInfo info;
 
     if (!gst_video_info_from_caps (&info, caps)) {
@@ -500,11 +499,13 @@ gst_d3dvideosink_propose_allocation (GstBaseSink * bsink, GstQuery * query)
       return FALSE;
     }
 
-    GST_DEBUG_OBJECT (sink, "create new pool");
-    pool = gst_d3dsurface_buffer_pool_new (sink);
-
     /* the normal size of a frame */
     size = info.size;
+  }
+
+  if (pool == NULL && need_pool) {
+    GST_DEBUG_OBJECT (sink, "create new pool");
+    pool = gst_d3dsurface_buffer_pool_new (sink);
 
     config = gst_buffer_pool_get_config (pool);
     /* we need at least 2 buffer because we hold on to the last one */
@@ -516,16 +517,15 @@ gst_d3dvideosink_propose_allocation (GstBaseSink * bsink, GstQuery * query)
     }
   }
 
-  if (pool) {
-    /* we need at least 2 buffer because we hold on to the last one */
-    gst_query_add_allocation_pool (query, pool, size, 2, 0);
+  /* we need at least 2 buffer because we hold on to the last one */
+  gst_query_add_allocation_pool (query, pool, size, 2, 0);
+  if (pool)
     gst_object_unref (pool);
-  }
 
   return TRUE;
 }
 
-/** PUBLIC FUNCTIONS **/
+/* PUBLIC FUNCTIONS */
 
 /* Iterface Registrations */
 
@@ -602,7 +602,7 @@ gst_d3dvideosink_navigation_send_event (GstNavigation * navigation,
   }
 }
 
-/** PRIVATE FUNCTIONS **/
+/* PRIVATE FUNCTIONS */
 
 
 /* Plugin entry point */
@@ -619,6 +619,6 @@ plugin_init (GstPlugin * plugin)
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    d3dsinkwrapper,
-    "Direct3D sink wrapper plugin",
+    d3d,
+    "Direct3D plugin",
     plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
