@@ -107,6 +107,11 @@ struct _GstMsdkDecClass
   GstVideoDecoderClass parent_class;
 
   gboolean (*configure) (GstMsdkDec * decoder);
+
+  /* reset mfx parameters per codec */
+  gboolean (*preinit_decoder) (GstMsdkDec * decoder);
+  /* adjust mfx parameters per codec */
+  gboolean (*postinit_decoder) (GstMsdkDec * decoder);
 };
 
 struct _MsdkDecTask
