@@ -214,7 +214,7 @@ gst_msdkvp8enc_class_init (GstMsdkVP8EncClass * klass)
 
   gst_element_class_set_static_metadata (element_class,
       "Intel MSDK VP8 encoder",
-      "Codec/Encoder/Video",
+      "Codec/Encoder/Video/Hardware",
       "VP8 video encoder based on Intel Media SDK",
       "Josep Torra <jtorra@oblong.com>");
 
