@@ -44,9 +44,10 @@ gst_mssdemux_http_src_start (GstTestHTTPSrc * src,
 {
   const GstMssDemuxTestInputData *input =
       (const GstMssDemuxTestInputData *) user_data;
+  guint i;
 
 
-  for (guint i = 0; input[i].uri; ++i) {
+  for (i = 0; input[i].uri; ++i) {
     if (strcmp (input[i].uri, uri) == 0) {
       input_data->context = (gpointer) & input[i];
       input_data->size = input[i].size;
@@ -78,11 +79,12 @@ gst_mssdemux_http_src_create (GstTestHTTPSrc * src,
   } else {
     GstMapInfo info;
     guint pattern;
+    guint64 i;
 
     pattern = offset - offset % sizeof (pattern);
 
     gst_buffer_map (buf, &info, GST_MAP_WRITE);
-    for (guint64 i = 0; i < length; ++i) {
+    for (i = 0; i < length; ++i) {
       gchar pattern_byte_to_write = (offset + i) % sizeof (pattern);
       if (pattern_byte_to_write == 0) {
         pattern = offset + i;
@@ -311,7 +313,8 @@ GST_START_TEST (testSeekSnapBeforePosition)
 {
   /* Seek to 1.5s, snap before, it go to 1s */
   run_seek_position_test (1.0, GST_SEEK_TYPE_SET, 1500 * GST_MSECOND,
-      GST_SEEK_TYPE_NONE, 0, GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_SNAP_BEFORE,
+      GST_SEEK_TYPE_NONE, 0,
+      GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT | GST_SEEK_FLAG_SNAP_BEFORE,
       1000 * GST_MSECOND, -1, 3);
 }
 
@@ -322,7 +325,8 @@ GST_START_TEST (testSeekSnapAfterPosition)
 {
   /* Seek to 1.5s with snap after, it should move to 2s */
   run_seek_position_test (1.0, GST_SEEK_TYPE_SET, 1500 * GST_MSECOND,
-      GST_SEEK_TYPE_NONE, 0, GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_SNAP_AFTER,
+      GST_SEEK_TYPE_NONE, 0,
+      GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT | GST_SEEK_FLAG_SNAP_AFTER,
       2000 * GST_MSECOND, -1, 2);
 }
 
@@ -333,8 +337,8 @@ GST_START_TEST (testReverseSeekSnapBeforePosition)
 {
   run_seek_position_test (-1.0, GST_SEEK_TYPE_SET, 1000 * GST_MSECOND,
       GST_SEEK_TYPE_SET, 2500 * GST_MSECOND,
-      GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_SNAP_BEFORE, 1000 * GST_MSECOND,
-      3000 * GST_MSECOND, 2);
+      GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT | GST_SEEK_FLAG_SNAP_BEFORE,
+      1000 * GST_MSECOND, 3000 * GST_MSECOND, 2);
 }
 
 GST_END_TEST;
@@ -344,8 +348,8 @@ GST_START_TEST (testReverseSeekSnapAfterPosition)
 {
   run_seek_position_test (-1.0, GST_SEEK_TYPE_SET, 1000 * GST_MSECOND,
       GST_SEEK_TYPE_SET, 2500 * GST_MSECOND,
-      GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_SNAP_AFTER, 1000 * GST_MSECOND,
-      2000 * GST_MSECOND, 1);
+      GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT | GST_SEEK_FLAG_SNAP_AFTER,
+      1000 * GST_MSECOND, 2000 * GST_MSECOND, 1);
 }
 
 GST_END_TEST;
